@@ -1,5 +1,8 @@
 use derive_more::{Display, Error};
-use std::{num::ParseIntError, str::FromStr};
+use std::{
+    num::{NonZeroUsize, ParseIntError},
+    str::FromStr,
+};
 
 const AUTO: &str = "auto";
 const MAX: &str = "max";
@@ -12,11 +15,12 @@ pub enum Threads {
     Auto,
     #[display("{MAX}")]
     Max,
-    Fixed(usize),
+    Fixed(NonZeroUsize),
 }
 
-/// Error that occurs when converting a string to an instance of [`Threads`].
+/// Error that occurs when parsing a string to as [`Threads`].
 #[derive(Debug, Display, Clone, PartialEq, Eq, Error)]
+#[non_exhaustive]
 pub enum FromStrError {
     #[display("Value is neither {AUTO:?}, {MAX:?}, nor a number: {_0}")]
     InvalidSyntax(ParseIntError),
@@ -24,15 +28,14 @@ pub enum FromStrError {
 
 impl FromStr for Threads {
     type Err = FromStrError;
-    fn from_str(value: &str) -> Result<Self, Self::Err> {
-        let value = value.trim();
-        match value {
+    fn from_str(text: &str) -> Result<Self, Self::Err> {
+        let text = text.trim();
+        match text {
             AUTO => return Ok(Threads::Auto),
             MAX => return Ok(Threads::Max),
             _ => {}
         };
-        value
-            .parse()
+        text.parse()
             .map_err(FromStrError::InvalidSyntax)
             .map(Threads::Fixed)
     }
