@@ -13,10 +13,11 @@ where
         self.children
             .par_iter_mut()
             .for_each(|child| child.par_sort_by(compare));
-        self.children.sort_by(compare);
+        self.children.sort_unstable_by(compare);
     }
 
     /// Process the tree via [`par_sort_by`](Self::par_sort_by) method.
+    #[inline]
     pub fn into_par_sorted(
         mut self,
         compare: impl Fn(&Self, &Self) -> Ordering + Copy + Sync,
