@@ -1,3 +1,8 @@
+//! The library crate for `pdu`.
+//!
+//! The things you are most interested in are likely [`fs_tree_builder::FsTreeBuilder`],
+//! [`tree_builder::TreeBuilder`], [`data_tree::DataTree`], or [`visualizer::Visualizer`].
+
 #![deny(warnings)]
 
 #[cfg(feature = "json")]
@@ -17,7 +22,7 @@ pub mod runtime_error;
 pub fn main() -> std::process::ExitCode {
     if let Err(error) = app::App::from_env().run() {
         eprintln!("[error] {error}");
-        return std::process::ExitCode::FAILURE;
+        return error.code();
     }
     std::process::ExitCode::SUCCESS
 }
@@ -33,6 +38,8 @@ pub mod bytes_format;
 pub mod data_tree;
 pub mod fs_tree_builder;
 pub mod get_size;
+pub mod hardlink;
+pub mod inode;
 pub mod json_data;
 pub mod os_string_display;
 pub mod reporter;
