@@ -8,17 +8,18 @@ use parallel_disk_usage::{
     bytes_format::BytesFormat,
     data_tree::DataTree,
     fs_tree_builder::FsTreeBuilder,
-    get_size::{GetApparentSize, GetBlockCount, GetBlockSize},
+    get_size::GetApparentSize,
+    hardlink::HardlinkIgnorant,
     os_string_display::OsStringDisplay,
     reporter::{ErrorOnlyReporter, ErrorReport},
     visualizer::{BarAlignment, ColumnWidthDistribution, Direction, Visualizer},
 };
 use pipe_trait::Pipe;
 use pretty_assertions::assert_eq;
-use std::{
-    convert::TryInto,
-    process::{Command, Stdio},
-};
+use std::process::{Command, Stdio};
+
+#[cfg(unix)]
+use parallel_disk_usage::get_size::{GetBlockCount, GetBlockSize};
 
 fn stdio(command: Command) -> Command {
     command
@@ -42,7 +43,9 @@ fn total_width() {
     let builder = FsTreeBuilder {
         root: workspace.to_path_buf(),
         size_getter: DEFAULT_GET_SIZE,
-        reporter: ErrorOnlyReporter::new(ErrorReport::SILENT),
+        hardlinks_recorder: &HardlinkIgnorant,
+        reporter: &ErrorOnlyReporter::new(ErrorReport::SILENT),
+        max_depth: 10,
     };
     let mut data_tree: DataTree<OsStringDisplay, _> = builder.into();
     data_tree.par_cull_insignificant_data(0.01);
@@ -54,7 +57,6 @@ fn total_width() {
         direction: Direction::BottomUp,
         bar_alignment: BarAlignment::Left,
         column_width_distribution: ColumnWidthDistribution::total(100),
-        max_depth: 10.try_into().unwrap(),
     };
     let expected = format!("{visualizer}");
     let expected = expected.trim_end();
@@ -80,7 +82,9 @@ fn column_width() {
     let builder = FsTreeBuilder {
         root: workspace.to_path_buf(),
         size_getter: DEFAULT_GET_SIZE,
-        reporter: ErrorOnlyReporter::new(ErrorReport::SILENT),
+        hardlinks_recorder: &HardlinkIgnorant,
+        reporter: &ErrorOnlyReporter::new(ErrorReport::SILENT),
+        max_depth: 10,
     };
     let mut data_tree: DataTree<OsStringDisplay, _> = builder.into();
     data_tree.par_cull_insignificant_data(0.01);
@@ -92,7 +96,6 @@ fn column_width() {
         direction: Direction::BottomUp,
         bar_alignment: BarAlignment::Left,
         column_width_distribution: ColumnWidthDistribution::components(10, 90),
-        max_depth: 10.try_into().unwrap(),
     };
     let expected = format!("{visualizer}");
     let expected = expected.trim_end();
@@ -118,7 +121,9 @@ fn min_ratio_0() {
     let builder = FsTreeBuilder {
         root: workspace.to_path_buf(),
         size_getter: GetApparentSize,
-        reporter: ErrorOnlyReporter::new(ErrorReport::SILENT),
+        hardlinks_recorder: &HardlinkIgnorant,
+        reporter: &ErrorOnlyReporter::new(ErrorReport::SILENT),
+        max_depth: 10,
     };
     let mut data_tree: DataTree<OsStringDisplay, _> = builder.into();
     data_tree.par_sort_by(|left, right| left.size().cmp(&right.size()).reverse());
@@ -129,7 +134,6 @@ fn min_ratio_0() {
         direction: Direction::BottomUp,
         bar_alignment: BarAlignment::Left,
         column_width_distribution: ColumnWidthDistribution::total(100),
-        max_depth: 10.try_into().unwrap(),
     };
     let expected = format!("{visualizer}");
     let expected = expected.trim_end();
@@ -155,7 +159,9 @@ fn min_ratio() {
     let builder = FsTreeBuilder {
         root: workspace.to_path_buf(),
         size_getter: GetApparentSize,
-        reporter: ErrorOnlyReporter::new(ErrorReport::SILENT),
+        hardlinks_recorder: &HardlinkIgnorant,
+        reporter: &ErrorOnlyReporter::new(ErrorReport::SILENT),
+        max_depth: 10,
     };
     let mut data_tree: DataTree<OsStringDisplay, _> = builder.into();
     data_tree.par_cull_insignificant_data(0.1);
@@ -167,7 +173,6 @@ fn min_ratio() {
         direction: Direction::BottomUp,
         bar_alignment: BarAlignment::Left,
         column_width_distribution: ColumnWidthDistribution::total(100),
-        max_depth: 10.try_into().unwrap(),
     };
     let expected = format!("{visualizer}");
     let expected = expected.trim_end();
@@ -193,7 +198,9 @@ fn max_depth_2() {
     let builder = FsTreeBuilder {
         root: workspace.to_path_buf(),
         size_getter: GetApparentSize,
-        reporter: ErrorOnlyReporter::new(ErrorReport::SILENT),
+        hardlinks_recorder: &HardlinkIgnorant,
+        reporter: &ErrorOnlyReporter::new(ErrorReport::SILENT),
+        max_depth: 2,
     };
     let mut data_tree: DataTree<OsStringDisplay, _> = builder.into();
     data_tree.par_cull_insignificant_data(0.01);
@@ -205,7 +212,6 @@ fn max_depth_2() {
         direction: Direction::BottomUp,
         bar_alignment: BarAlignment::Left,
         column_width_distribution: ColumnWidthDistribution::total(100),
-        max_depth: 2.try_into().unwrap(),
     };
     let expected = format!("{visualizer}");
     let expected = expected.trim_end();
@@ -231,7 +237,9 @@ fn max_depth_1() {
     let builder = FsTreeBuilder {
         root: workspace.to_path_buf(),
         size_getter: GetApparentSize,
-        reporter: ErrorOnlyReporter::new(ErrorReport::SILENT),
+        hardlinks_recorder: &HardlinkIgnorant,
+        reporter: &ErrorOnlyReporter::new(ErrorReport::SILENT),
+        max_depth: 1,
     };
     let mut data_tree: DataTree<OsStringDisplay, _> = builder.into();
     data_tree.par_cull_insignificant_data(0.01);
@@ -243,7 +251,6 @@ fn max_depth_1() {
         direction: Direction::BottomUp,
         bar_alignment: BarAlignment::Left,
         column_width_distribution: ColumnWidthDistribution::total(100),
-        max_depth: 1.try_into().unwrap(),
     };
     let expected = format!("{visualizer}");
     let expected = expected.trim_end();
@@ -268,7 +275,9 @@ fn top_down() {
     let builder = FsTreeBuilder {
         root: workspace.to_path_buf(),
         size_getter: DEFAULT_GET_SIZE,
-        reporter: ErrorOnlyReporter::new(ErrorReport::SILENT),
+        hardlinks_recorder: &HardlinkIgnorant,
+        reporter: &ErrorOnlyReporter::new(ErrorReport::SILENT),
+        max_depth: 10,
     };
     let mut data_tree: DataTree<OsStringDisplay, _> = builder.into();
     data_tree.par_cull_insignificant_data(0.01);
@@ -280,7 +289,6 @@ fn top_down() {
         direction: Direction::TopDown,
         bar_alignment: BarAlignment::Left,
         column_width_distribution: ColumnWidthDistribution::total(100),
-        max_depth: 10.try_into().unwrap(),
     };
     let expected = format!("{visualizer}");
     let expected = expected.trim_end();
@@ -305,7 +313,9 @@ fn align_right() {
     let builder = FsTreeBuilder {
         root: workspace.to_path_buf(),
         size_getter: DEFAULT_GET_SIZE,
-        reporter: ErrorOnlyReporter::new(ErrorReport::SILENT),
+        hardlinks_recorder: &HardlinkIgnorant,
+        reporter: &ErrorOnlyReporter::new(ErrorReport::SILENT),
+        max_depth: 10,
     };
     let mut data_tree: DataTree<OsStringDisplay, _> = builder.into();
     data_tree.par_cull_insignificant_data(0.01);
@@ -317,7 +327,6 @@ fn align_right() {
         direction: Direction::BottomUp,
         bar_alignment: BarAlignment::Right,
         column_width_distribution: ColumnWidthDistribution::total(100),
-        max_depth: 10.try_into().unwrap(),
     };
     let expected = format!("{visualizer}");
     let expected = expected.trim_end();
@@ -342,7 +351,9 @@ fn quantity_apparent_size() {
     let builder = FsTreeBuilder {
         root: workspace.to_path_buf(),
         size_getter: GetApparentSize,
-        reporter: ErrorOnlyReporter::new(ErrorReport::SILENT),
+        hardlinks_recorder: &HardlinkIgnorant,
+        reporter: &ErrorOnlyReporter::new(ErrorReport::SILENT),
+        max_depth: 10,
     };
     let mut data_tree: DataTree<OsStringDisplay, _> = builder.into();
     data_tree.par_cull_insignificant_data(0.01);
@@ -354,7 +365,6 @@ fn quantity_apparent_size() {
         direction: Direction::BottomUp,
         bar_alignment: BarAlignment::Left,
         column_width_distribution: ColumnWidthDistribution::total(100),
-        max_depth: 10.try_into().unwrap(),
     };
     let expected = format!("{visualizer}");
     let expected = expected.trim_end();
@@ -380,7 +390,9 @@ fn quantity_block_size() {
     let builder = FsTreeBuilder {
         root: workspace.to_path_buf(),
         size_getter: GetBlockSize,
-        reporter: ErrorOnlyReporter::new(ErrorReport::SILENT),
+        hardlinks_recorder: &HardlinkIgnorant,
+        reporter: &ErrorOnlyReporter::new(ErrorReport::SILENT),
+        max_depth: 10,
     };
     let mut data_tree: DataTree<OsStringDisplay, _> = builder.into();
     data_tree.par_cull_insignificant_data(0.01);
@@ -392,7 +404,6 @@ fn quantity_block_size() {
         direction: Direction::BottomUp,
         bar_alignment: BarAlignment::Left,
         column_width_distribution: ColumnWidthDistribution::total(100),
-        max_depth: 10.try_into().unwrap(),
     };
     let expected = format!("{visualizer}");
     let expected = expected.trim_end();
@@ -418,7 +429,9 @@ fn quantity_block_count() {
     let builder = FsTreeBuilder {
         root: workspace.to_path_buf(),
         size_getter: GetBlockCount,
-        reporter: ErrorOnlyReporter::new(ErrorReport::SILENT),
+        hardlinks_recorder: &HardlinkIgnorant,
+        reporter: &ErrorOnlyReporter::new(ErrorReport::SILENT),
+        max_depth: 10,
     };
     let mut data_tree: DataTree<OsStringDisplay, _> = builder.into();
     data_tree.par_cull_insignificant_data(0.01);
@@ -430,7 +443,6 @@ fn quantity_block_count() {
         direction: Direction::BottomUp,
         bar_alignment: BarAlignment::Left,
         column_width_distribution: ColumnWidthDistribution::total(100),
-        max_depth: 10.try_into().unwrap(),
     };
     let expected = format!("{visualizer}");
     let expected = expected.trim_end();
@@ -439,14 +451,13 @@ fn quantity_block_count() {
     assert_eq!(actual, expected);
 }
 
-#[cfg(unix)]
 #[test]
 fn bytes_format_plain() {
     let workspace = SampleWorkspace::default();
     let actual = Command::new(PDU)
         .with_current_dir(&workspace)
         .with_arg("--total-width=100")
-        .with_arg("--quantity=block-size")
+        .with_arg("--quantity=apparent-size")
         .with_arg("--bytes-format=plain")
         .pipe(stdio)
         .output()
@@ -456,8 +467,10 @@ fn bytes_format_plain() {
 
     let builder = FsTreeBuilder {
         root: workspace.to_path_buf(),
-        size_getter: GetBlockSize,
-        reporter: ErrorOnlyReporter::new(ErrorReport::SILENT),
+        size_getter: GetApparentSize,
+        hardlinks_recorder: &HardlinkIgnorant,
+        reporter: &ErrorOnlyReporter::new(ErrorReport::SILENT),
+        max_depth: 10,
     };
     let mut data_tree: DataTree<OsStringDisplay, _> = builder.into();
     data_tree.par_cull_insignificant_data(0.01);
@@ -469,7 +482,6 @@ fn bytes_format_plain() {
         direction: Direction::BottomUp,
         bar_alignment: BarAlignment::Left,
         column_width_distribution: ColumnWidthDistribution::total(100),
-        max_depth: 10.try_into().unwrap(),
     };
     let expected = format!("{visualizer}");
     let expected = expected.trim_end();
@@ -478,14 +490,13 @@ fn bytes_format_plain() {
     assert_eq!(actual, expected);
 }
 
-#[cfg(unix)]
 #[test]
 fn bytes_format_metric() {
     let workspace = SampleWorkspace::default();
     let actual = Command::new(PDU)
         .with_current_dir(&workspace)
         .with_arg("--total-width=100")
-        .with_arg("--quantity=block-size")
+        .with_arg("--quantity=apparent-size")
         .with_arg("--bytes-format=metric")
         .pipe(stdio)
         .output()
@@ -495,8 +506,10 @@ fn bytes_format_metric() {
 
     let builder = FsTreeBuilder {
         root: workspace.to_path_buf(),
-        size_getter: GetBlockSize,
-        reporter: ErrorOnlyReporter::new(ErrorReport::SILENT),
+        size_getter: GetApparentSize,
+        hardlinks_recorder: &HardlinkIgnorant,
+        reporter: &ErrorOnlyReporter::new(ErrorReport::SILENT),
+        max_depth: 10,
     };
     let mut data_tree: DataTree<OsStringDisplay, _> = builder.into();
     data_tree.par_cull_insignificant_data(0.01);
@@ -508,7 +521,6 @@ fn bytes_format_metric() {
         direction: Direction::BottomUp,
         bar_alignment: BarAlignment::Left,
         column_width_distribution: ColumnWidthDistribution::total(100),
-        max_depth: 10.try_into().unwrap(),
     };
     let expected = format!("{visualizer}");
     let expected = expected.trim_end();
@@ -517,14 +529,13 @@ fn bytes_format_metric() {
     assert_eq!(actual, expected);
 }
 
-#[cfg(unix)]
 #[test]
 fn bytes_format_binary() {
     let workspace = SampleWorkspace::default();
     let actual = Command::new(PDU)
         .with_current_dir(&workspace)
         .with_arg("--total-width=100")
-        .with_arg("--quantity=block-size")
+        .with_arg("--quantity=apparent-size")
         .with_arg("--bytes-format=binary")
         .pipe(stdio)
         .output()
@@ -534,8 +545,10 @@ fn bytes_format_binary() {
 
     let builder = FsTreeBuilder {
         root: workspace.to_path_buf(),
-        size_getter: GetBlockSize,
-        reporter: ErrorOnlyReporter::new(ErrorReport::SILENT),
+        size_getter: GetApparentSize,
+        hardlinks_recorder: &HardlinkIgnorant,
+        reporter: &ErrorOnlyReporter::new(ErrorReport::SILENT),
+        max_depth: 10,
     };
     let mut data_tree: DataTree<OsStringDisplay, _> = builder.into();
     data_tree.par_cull_insignificant_data(0.01);
@@ -547,7 +560,6 @@ fn bytes_format_binary() {
         direction: Direction::BottomUp,
         bar_alignment: BarAlignment::Left,
         column_width_distribution: ColumnWidthDistribution::total(100),
-        max_depth: 10.try_into().unwrap(),
     };
     let expected = format!("{visualizer}");
     let expected = expected.trim_end();
@@ -572,7 +584,9 @@ fn path_to_workspace() {
     let builder = FsTreeBuilder {
         root: workspace.to_path_buf(),
         size_getter: DEFAULT_GET_SIZE,
-        reporter: ErrorOnlyReporter::new(ErrorReport::SILENT),
+        hardlinks_recorder: &HardlinkIgnorant,
+        reporter: &ErrorOnlyReporter::new(ErrorReport::SILENT),
+        max_depth: 10,
     };
     let mut data_tree: DataTree<OsStringDisplay, _> = builder.into();
     data_tree.par_cull_insignificant_data(0.01);
@@ -583,7 +597,6 @@ fn path_to_workspace() {
         direction: Direction::BottomUp,
         bar_alignment: BarAlignment::Left,
         column_width_distribution: ColumnWidthDistribution::total(100),
-        max_depth: 10.try_into().unwrap(),
     };
     let expected = format!("{visualizer}");
     let expected = expected.trim_end();
@@ -614,7 +627,9 @@ fn multiple_names() {
             let builder = FsTreeBuilder {
                 root: workspace.to_path_buf().join(name),
                 size_getter: GetApparentSize,
-                reporter: ErrorOnlyReporter::new(ErrorReport::SILENT),
+                hardlinks_recorder: &HardlinkIgnorant,
+                reporter: &ErrorOnlyReporter::new(ErrorReport::SILENT),
+                max_depth: 10,
             };
             let mut data_tree: DataTree<OsStringDisplay, _> = builder.into();
             *data_tree.name_mut() = OsStringDisplay::os_string_from(name);
@@ -635,11 +650,140 @@ fn multiple_names() {
         direction: Direction::BottomUp,
         bar_alignment: BarAlignment::Left,
         column_width_distribution: ColumnWidthDistribution::total(100),
-        max_depth: 10.try_into().unwrap(),
     };
     let expected = format!("{visualizer}");
     let expected = expected.trim_end();
     eprintln!("EXPECTED:\n{expected}\n");
 
     assert_eq!(actual, expected);
+
+    let mut lines = actual.lines();
+    assert!(lines.next().unwrap().contains("┌──1"));
+    assert!(lines.next().unwrap().contains("┌─┴0"));
+    assert!(lines.next().unwrap().contains("┌─┴nested"));
+    assert!(lines.next().unwrap().contains("│ ┌──1"));
+    assert!(lines.next().unwrap().contains("│ ├──2"));
+    assert!(lines.next().unwrap().contains("│ ├──3"));
+    assert!(lines.next().unwrap().contains("├─┴flat"));
+    assert!(lines.next().unwrap().contains("┌─┴(total)"));
+    assert_eq!(lines.next(), None);
+}
+
+#[test]
+fn multiple_names_max_depth_2() {
+    let workspace = SampleWorkspace::default();
+    let actual = Command::new(PDU)
+        .with_current_dir(&workspace)
+        .with_arg("--quantity=apparent-size")
+        .with_arg("--total-width=100")
+        .with_arg("--max-depth=2")
+        .with_arg("nested")
+        .with_arg("flat")
+        .with_arg("empty-dir")
+        .pipe(stdio)
+        .output()
+        .expect("spawn command")
+        .pipe(stdout_text);
+    eprintln!("ACTUAL:\n{actual}\n");
+
+    let mut data_tree = ["nested", "flat", "empty-dir"]
+        .iter()
+        .map(|name| {
+            let builder = FsTreeBuilder {
+                root: workspace.to_path_buf().join(name),
+                size_getter: GetApparentSize,
+                hardlinks_recorder: &HardlinkIgnorant,
+                reporter: &ErrorOnlyReporter::new(ErrorReport::SILENT),
+                max_depth: 1,
+            };
+            let mut data_tree: DataTree<OsStringDisplay, _> = builder.into();
+            *data_tree.name_mut() = OsStringDisplay::os_string_from(name);
+            data_tree
+        })
+        .pipe(|children| {
+            DataTree::dir(
+                OsStringDisplay::os_string_from("(total)"),
+                0.into(),
+                children.collect(),
+            )
+        })
+        .into_par_sorted(|left, right| left.size().cmp(&right.size()).reverse());
+    data_tree.par_cull_insignificant_data(0.01);
+    let visualizer = Visualizer::<OsStringDisplay, _> {
+        data_tree: &data_tree,
+        bytes_format: BytesFormat::MetricUnits,
+        direction: Direction::BottomUp,
+        bar_alignment: BarAlignment::Left,
+        column_width_distribution: ColumnWidthDistribution::total(100),
+    };
+    let expected = format!("{visualizer}");
+    let expected = expected.trim_end();
+    eprintln!("EXPECTED:\n{expected}\n");
+
+    assert_eq!(actual, expected);
+
+    let mut lines = actual.lines();
+    assert!(lines.next().unwrap().contains("┌──nested"));
+    assert!(lines.next().unwrap().contains("├──flat"));
+    assert!(lines.next().unwrap().contains("┌─┴(total)"));
+    assert_eq!(lines.next(), None);
+}
+
+#[test]
+fn multiple_names_max_depth_1() {
+    let workspace = SampleWorkspace::default();
+    let actual = Command::new(PDU)
+        .with_current_dir(&workspace)
+        .with_arg("--quantity=apparent-size")
+        .with_arg("--total-width=100")
+        .with_arg("--max-depth=1")
+        .with_arg("nested")
+        .with_arg("flat")
+        .with_arg("empty-dir")
+        .pipe(stdio)
+        .output()
+        .expect("spawn command")
+        .pipe(stdout_text);
+    eprintln!("ACTUAL:\n{actual}\n");
+
+    let mut data_tree = ["nested", "flat", "empty-dir"]
+        .iter()
+        .map(|name| {
+            let builder = FsTreeBuilder {
+                root: workspace.to_path_buf().join(name),
+                size_getter: GetApparentSize,
+                hardlinks_recorder: &HardlinkIgnorant,
+                reporter: &ErrorOnlyReporter::new(ErrorReport::SILENT),
+                max_depth: 10,
+            };
+            let mut data_tree: DataTree<OsStringDisplay, _> = builder.into();
+            *data_tree.name_mut() = OsStringDisplay::os_string_from(name);
+            data_tree
+        })
+        .pipe(|children| {
+            DataTree::dir(
+                OsStringDisplay::os_string_from("(total)"),
+                0.into(),
+                children.collect(),
+            )
+        })
+        .into_par_retained(|_, _| false)
+        .into_par_sorted(|left, right| left.size().cmp(&right.size()).reverse());
+    data_tree.par_cull_insignificant_data(0.01);
+    let visualizer = Visualizer::<OsStringDisplay, _> {
+        data_tree: &data_tree,
+        bytes_format: BytesFormat::MetricUnits,
+        direction: Direction::BottomUp,
+        bar_alignment: BarAlignment::Left,
+        column_width_distribution: ColumnWidthDistribution::total(100),
+    };
+    let expected = format!("{visualizer}");
+    let expected = expected.trim_end();
+    eprintln!("EXPECTED:\n{expected}\n");
+
+    assert_eq!(actual, expected);
+
+    let mut lines = actual.lines();
+    assert!(lines.next().unwrap().contains("┌──(total)"));
+    assert_eq!(lines.next(), None);
 }
